#include <stdio.h>
#include "esp_log.h"
#include "driver/i2c.h"
#include "gw_grayscale_sensor.h"

static const char *TAG = "gw gray i2c example";

/* 使用传感器自带的模拟量归一化(v3.6及之后的固件) */
#define USE_GW_GRAY_NORMALIZE 1

#define I2C_MASTER_SCL_IO           22      /*!< I2C 时钟针脚号  */
#define I2C_MASTER_SDA_IO           21      /*!< I2C 数据针脚号  */
#define I2C_MASTER_NUM              0       /*!< I2C 通道       */
#define I2C_MASTER_FREQ_HZ          100000  /*!< I2C 主的时钟频率 */
#define I2C_MASTER_TX_BUF_DISABLE   0       /*!< I2C 不设输出缓冲 */
#define I2C_MASTER_RX_BUF_DISABLE   0       /*!< I2C 不设输入缓冲 */
#define I2C_MASTER_TIMEOUT_MS       0

static const uint8_t gw_gray_addr = GW_GRAY_ADDR_DEF; /*! 使用默认地址 */

static esp_err_t gw_gray_write_cmd(uint8_t cmd)
{
	esp_err_t err;
	err = i2c_master_write_to_device(I2C_MASTER_NUM, gw_gray_addr,
							   &cmd, 1, pdMS_TO_TICKS(I2C_MASTER_TIMEOUT_MS));
	return err;
}

static esp_err_t gw_gray_read_byte(uint8_t *byte_read)
{
	esp_err_t err;
	err = i2c_master_read_from_device(I2C_MASTER_NUM, gw_gray_addr,
								byte_read, 1, pdMS_TO_TICKS(I2C_MASTER_TIMEOUT_MS));
	return err;
}

static esp_err_t gw_gray_read(uint8_t *read_arr, size_t read_size)
{
	esp_err_t err;
	err = i2c_master_read_from_device(I2C_MASTER_NUM, gw_gray_addr,
	                                  read_arr, read_size, pdMS_TO_TICKS(I2C_MASTER_TIMEOUT_MS));
	return err;
}

static esp_err_t i2c_master_init(void)
{
    int i2c_master_port = I2C_MASTER_NUM;

    i2c_config_t conf = {
        .mode = I2C_MODE_MASTER,
        .sda_io_num = I2C_MASTER_SDA_IO,
        .scl_io_num = I2C_MASTER_SCL_IO,
        .sda_pullup_en = GPIO_PULLUP_ENABLE,
        .scl_pullup_en = GPIO_PULLUP_ENABLE,
        .master.clk_speed = I2C_MASTER_FREQ_HZ,
    };

    i2c_param_config(i2c_master_port, &conf);

    return i2c_driver_install(i2c_master_port, conf.mode, I2C_MASTER_RX_BUF_DISABLE, I2C_MASTER_TX_BUF_DISABLE, 0);
}


void app_main(void)
{
	uint8_t ping_rsp = 0;
	uint8_t data_arr[8] = {0};
	uint8_t write_buf[2];
	int err = 0;
    ESP_ERROR_CHECK(i2c_master_init());
    ESP_LOGI(TAG, "I2C initialized successfully");

	/* PING 通为止 */
	while (ping_rsp != GW_GRAY_PING_OK) {
		ping_rsp = 0;
		err = gw_gray_write_cmd(GW_GRAY_PING);
		err = gw_gray_read_byte(&ping_rsp);
		if (err != ESP_OK) {
			ESP_LOGE(TAG, "%s", esp_err_to_name(err));
		}
		ESP_LOGI(TAG, "ping_rsp = %X", ping_rsp);
		vTaskDelay(pdMS_TO_TICKS(500));
	}
	ESP_LOGI(TAG, "PING OK");

#if (USE_GW_GRAY_NORMALIZE)
	/* 使用传感器内部自带归一化(v3.6及之后的固件) */
	write_buf[0] = GW_GRAY_ANALOG_NORMALIZE;
	write_buf[1] = 0xFF; // 所有通道归一化
	err = i2c_master_write_to_device(I2C_MASTER_NUM, gw_gray_addr,
									 write_buf, 2, pdMS_TO_TICKS(I2C_MASTER_TIMEOUT_MS));
	ESP_ERROR_CHECK_WITHOUT_ABORT(err);
#endif
	/* 模拟数据模式, 依次读取8个探头数据 */
	err = gw_gray_write_cmd(GW_GRAY_ANALOG_MODE);
	ESP_ERROR_CHECK_WITHOUT_ABORT(err);

	while (1) {
		err = gw_gray_read(data_arr, 8);
		for (int i = 0; i < 8; ++i) {
			printf("[%d]: %d  |", i, data_arr[i]);
		}
		printf("\n");
		vTaskDelay(pdMS_TO_TICKS(100));
	}
}
